@@ -4,8 +4,9 @@ use v5.10;
 use LWP::UserAgent;
 use Scalar::Util qw(looks_like_number);
 use Carp;
+use JSON;
 use XML::LibXML;
-use XML::Hash::XS qw();
+use URI::Escape;
 use Moo::Role;
 
 =head1 NAME
@@ -14,11 +15,11 @@ Ovirt - Bindings for oVirt REST API
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -32,6 +33,7 @@ our $VERSION = '0.04';
             username                => 'admin',
             password                => 'password',
             manager                 => 'ovirt-mgr.example.com',
+            api_output              => 'xml', # optional, default to json
             vm_output_attrs         => 'id,name,state,description', # optional
             cluster_output_attrs    => 'id,name,cpu_id,cpu_arch,description', # optional
  );
@@ -41,13 +43,11 @@ our $VERSION = '0.04';
  my $template   = Ovirt::Template   ->new(%con);
  my $host       = Ovirt::Host       ->new(%con);
 
- # return xml output
- print $vm      ->list_xml;   
- print $cluster ->list_xml;
- print $template->list_xml;
- print $host    ->list_xml;                           
+ # return xml output / json
+ print $vm->list_xml;   
+ print $vm->list_json;                           
 
- # list attributes
+ # list attributes based on 'vm_output_attrs'
  print $vm      ->list;
  print $cluster ->list;
  print $template->list;
@@ -58,6 +58,19 @@ our $VERSION = '0.04';
  $vm->remove('2d83bb51-9a77-432d-939c-35be207017b9');
  
  # add/remove/list vm's nic and disk
+ $vm->add_disk('6efc0cfa-8495-4a96-93e5-ee490328cf48',  # vm id
+                'virtio',                               # driver interface
+                'cow',                                  # format
+                '1073741824',                           # size
+                'mydisk1',                              # disk name
+                '9b952bdc-b7ec-4673-84b0-477b48945a9a'  # storage domain id
+              );
+
+ $vm->add_nic('6efc0cfa-8495-4a96-93e5-ee490328cf48',   # vm id
+              'virtio',                                 # driver interface
+              'nic1',                                   # nic name
+              'rhevm'                                   # network name
+             );
  
  # start, stop, reboot, migrate vm
  $vm->start     ('b4738b0f-b73d-4a66-baa8-2ba465d63132');
@@ -125,18 +138,20 @@ our $VERSION = '0.04';
  ssl_verify     = (ro) disable host verification (default is no)
  log_severity   = (ro) store log severity level, valid value ERROR|OFF|FATAL|INFO|DEBUG|TRACE|ALL|WARN
                   (default is INFO)
+ api_output     = (ro) json/xml, default is json
  not_available  = (rw) store undef or empty output string, default to 'N/A'
  url            = (rwp) store final url to be requested to Ovirt
  root_url       = (rwp) store url on each object
  log            = (rwp) store log from log4perl
- xml_output     = (rwp) store xml output from Ovirt RestAPI
+ xml_output     = (rwp) store xml output from API output
+ json_output    = (rwp) store json output from API output
  hash_output    = (rwp) store hash output converted from xml output
 
 =cut
 
-has [qw/url root_url xml_output hash_output log/]   => ( is => 'rwp' );
-has [qw/id/]                                        => ( is => 'ro' );
-has [qw/username password manager/]                 => ( is => 'ro', required => 1 );
+has [qw/url root_url xml_output json_output hash_output log/]   => ( is => 'rwp' );
+has [qw/id/]                                                    => ( is => 'ro' );
+has [qw/username password manager/]                             => ( is => 'ro', required => 1 );
 
 has 'port'          => ( is => 'ro', 
                         isa => 
@@ -172,6 +187,12 @@ has 'log_severity'  => (is => 'ro',
                         default => 'INFO'
 );
 
+has 'api_output'    => (is => 'ro', 
+                        isa => sub { croak "valid api_output is xml/json\n" 
+                                        unless $_[0] =~ /(xml|json)/;
+                                    }, 
+                        default => 'json'
+);
 
 =head1 SUBROUTINES/METHODS
 
@@ -190,7 +211,8 @@ has 'log_severity'  => (is => 'ro',
  The Constructor, build logging, call pass_log_obj method
 =cut
 
-sub BUILD {
+sub BUILD 
+{
     my $self = shift;
     
     $self->pass_log_obj();
@@ -213,7 +235,8 @@ sub BUILD {
  
 =cut
 
-sub pass_log_obj {
+sub pass_log_obj 
+{
     my $self    = shift;
     
     # skip if already set
@@ -240,15 +263,21 @@ sub pass_log_obj {
  return the base url
 =cut
 
-sub base_url {
+sub base_url 
+{
     my $self = shift;
     
+    # make sure the http encoding
     # '%40' is '@'
-    my $url = $self->username . '%40'. $self->domain . ":" .$self->password . 
+    my $username = uri_escape($self->username);
+    my $domain   = uri_escape($self->domain);
+    my $password = uri_escape($self->password);
+    
+    my $url = $username . '%40'. $domain . ":" .$password . 
                 "\@" . $self->manager;
     
     if ($self->port) {
-        $url = $self->username . '%40'. $self->domain . ":" .$self->password . 
+        $url = $username . '%40'. $domain . ":" .$password . 
                 "\@" . $self->manager . ":" . $self->port;
     }
     
@@ -268,7 +297,8 @@ sub base_url {
  build the final url
 =cut
 
-sub api_url {
+sub api_url 
+{
     my $self = shift;
     
     # root_url is being set in each particular library
@@ -288,32 +318,196 @@ sub api_url {
  parameter. 
 =cut
 
-sub get_api_response {
+sub get_api_response 
+{
     my $self    = shift;
     
     my $ua      = LWP::UserAgent->new();
-    my $tx      = $ua->get($self->api_url);
     
-    if ($tx->is_success) {
+    if ($self->api_output eq 'json') {
         
-        local $XML::LibXML::skipXMLDeclaration = 1;
-        my $parser = XML::LibXML->new();
-        my $xml_string  = $parser->parse_string($tx->decoded_content);
-        $self->_set_xml_output($xml_string);
-        
-        #store to hash
-        my $conv    = XML::Hash::XS->new(utf8 => 1, encoding => 'utf8');
-        my $hash    = $conv->xml2hash($xml_string->toString, encoding => 'cp1251');
-        $self->_set_hash_output($hash);
+        my $tx  = $self->get_json_response($self->api_url);
         
+        if ($tx->is_success) {
+            #store json output
+            my $json = $tx->decoded_content;
+            $self->_set_json_output($json);
+            
+            my $hash    = from_json($json);
+            $self->_set_hash_output($hash);
+        }
+        else {
+            $self->set_lwp_error($tx);
+        }
     }
-    else {
-        my $err = $tx->status_line;
-        $self->log->debug("LWP Error : " . $err);
-        $self->log->debug("LWP Decoded Content :" . $tx->decoded_content);
+    elsif ($self->api_output eq 'xml') {
+        my $tx      = $self->get_xml_response($self->api_url);
         
-        croak "LWP Status line : " . $err;
-        croak "LWP Decoded Content :" . $tx->decoded_content;
+        if ($tx->is_success) {
+            use XML::Hash::XS qw();
+            
+            local $XML::LibXML::skipXMLDeclaration = 1;
+            my $parser = XML::LibXML->new();
+            my $xml_string  = $parser->parse_string($tx->decoded_content);
+            $self->_set_xml_output($xml_string);
+            
+            #store to hash
+            my $conv    = XML::Hash::XS->new(utf8 => 1, encoding => 'utf8');
+            my $hash    = $conv->xml2hash($xml_string->toString, encoding => 'cp1251');
+            $self->_set_hash_output($hash);
+            
+        }
+        else {
+            $self->set_lwp_error($tx);
+        }
+    }     
+}
+
+=head2 set_lwp_error
+
+ handle lwp if not success
+ required args ($tx)
+ 
+=cut
+
+sub set_lwp_error
+{
+    my $self    = shift;
+    my $tx      = shift;
+    
+    my $err = $tx->status_line;
+    $self->log->debug("LWP Error : " . $err);
+    $self->log->debug("LWP Decoded Content :" . $tx->decoded_content);
+    
+    croak "LWP Status line : " . $err;
+    croak "LWP Decoded Content :" . $tx->decoded_content;
+}
+
+=head2 get_api_output
+
+ handle and return output based on $self->api_output
+
+=cut
+
+sub get_api_output
+{
+    my $self    = shift;
+    my $url     = shift;
+    
+    if ($self->api_output eq 'json')
+    {
+        my $tx = $self->get_json_response($url);
+        if ($tx->is_success) 
+        {
+            return $tx->decoded_content;
+        }
+        else
+        {
+            $self->set_lwp_error($tx);  
+        }
+    }
+    else
+    {
+        my $tx = $self->get_xml_response($url);
+        
+        if ($tx->is_success) 
+        {
+            local $XML::LibXML::skipXMLDeclaration = 1;
+            my $parser      = XML::LibXML->new();
+            my $xml_string  = $parser->parse_string($tx->decoded_content);
+            return $xml_string;
+        }
+        else
+        {
+            $self->set_lwp_error($tx);
+        }
+    }
+}
+
+=head2 get_json_response
+
+ handle http get for json output
+ required argument ('url')
+ 
+=cut
+
+sub get_json_response 
+{
+    my $self    = shift;
+    
+    my $url     = shift;
+    
+    croak "url required"
+        unless $url;
+        
+    my $ua      = LWP::UserAgent->new();   
+    my $req     = HTTP::Request->new(GET => $url);
+    $req->header('content-type' => 'application/json');
+    $req->header('Accept' => 'application/json'); 
+    
+    my $tx  = $ua->request($req);
+    return $tx;     
+}
+
+=head2 get_xml_response
+
+ handle http get for xml output
+ required argument ('url')
+ 
+=cut
+
+sub get_xml_response 
+{
+    my $self    = shift;
+    
+    my $url     = shift;
+    
+    croak "url required"
+        unless $url;
+    
+    my $ua      = LWP::UserAgent->new();  
+    my $tx      = $ua->get($url);
+    
+    return $tx;     
+}
+
+=head2 list_xml
+
+ return xml output
+=cut
+
+sub list_xml 
+{
+    my $self = shift;
+    
+    if ($self->api_output eq 'xml') 
+    {
+        return $self->xml_output 
+            if $self->xml_output;
+    }
+    else 
+    {
+        die "api output setting is not xml\n";
+    }
+}
+
+=head2 list_json
+
+ return json output
+=cut
+
+sub list_json 
+{
+    my $self = shift;
+    
+    if ($self->api_output eq 'json') 
+    {
+        return $self->json_output
+            if $self->json_output;
+    }
+    else 
+    {
+        die "api output setting is not json\n";
     }
 }
 
